@@ -27,7 +27,7 @@
 #if !(defined(SDL_PLATFORM_XBOXONE) || defined(SDL_PLATFORM_XBOXSERIES))
 HICON CreateIconFromSurface(SDL_Surface *surface)
 {
-    SDL_Surface *s = SDL_ConvertSurface(surface, SDL_PIXELFORMAT_RGBA32);
+    SDL_Surface *s = SDL_ConvertSurface(surface, SDL_PIXELFORMAT_ARGB8888);
     if (!s) {
         return NULL;
     }
