@@ -133,7 +133,7 @@ perspective_matrix(float fovy, float aspect, float znear, float zfar, float *r)
     int i;
     float f;
 
-    f = 1.0f/SDL_tanf(fovy * 0.5f);
+    f = 1.0f/SDL_tanf((fovy / 180.0f) * SDL_PI_F  * 0.5f);
 
     for (i = 0; i < 16; i++) {
         r[i] = 0.0;
